//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Data/RangeUtil.cpp
//! @brief     Implements namespace RangeUtil.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Data/RangeUtil.h"
#include "GUI/Model/Axis/AmplitudeAxisItem.h"
#include "GUI/Model/Data/Data1DItem.h"
#include "GUI/Model/Data/Data2DItem.h"
#include <utility>

namespace {

double commonMin(const QList<AmplitudeAxisItem*>& axes)
{
    double min = +std::numeric_limits<double>::max();
    for (auto* axis : axes)
        if (min > axis->min())
            min = axis->min();
    return min;
}

double commonMax(const QList<AmplitudeAxisItem*>& axes)
{
    double max = -std::numeric_limits<double>::max();
    for (auto* axis : axes)
        if (max < axis->max())
            max = axis->max();
    return max;
}

std::pair<double, double> commonRange(const QList<AmplitudeAxisItem*>& axes)
{
    return {commonMin(axes), commonMax(axes)};
}

QList<AmplitudeAxisItem*> valueAxesFromData1DItems(const QList<Data1DItem*>& items)
{
    QList<AmplitudeAxisItem*> axes;
    for (auto* item : items)
        axes.append(item->axItemY());
    return axes;
}

QList<AmplitudeAxisItem*> valueAxesFromData2DItems(const QList<Data2DItem*>& items)
{
    QList<AmplitudeAxisItem*> axes;
    for (auto* item : items)
        axes.append(item->zAxisItem());
    return axes;
}

} // namespace

void GUI::Util::Ranges::setCommonRangeY(QList<Data1DItem*> items)
{
    std::pair<double, double> range = commonRange(valueAxesFromData1DItems(items));
    for (auto* item : items)
        item->setYrange(range.first, range.second);
}

void GUI::Util::Ranges::setCommonRangeZ(QList<Data2DItem*> items)
{
    std::pair<double, double> range = commonRange(valueAxesFromData2DItems(items));
    for (auto* item : items)
        item->setZrange(range.first, range.second);
}
